import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_TAG_SEARCH = "mythic_rpc_tag_search"


class MythicRPCTagSearchMessage:
    def __init__(self,
                 TaskID: int,
                 SearchTagID: int = None,
                 SearchTagTaskID: int = None,
                 SearchTagFileID: int = None,
                 SearchTagCredentialID: int = None,
                 SearchTagMythicTreeID: int = None,
                 SearchTagSource: str = None,
                 SearchTagData: str = None,
                 SearchTagURL: str = None,
                 **kwargs):
        self.TaskID = TaskID
        self.SearchTagID = SearchTagID
        self.SearchTagTaskID = SearchTagTaskID
        self.SearchTagFileID = SearchTagFileID
        self.SearchTagCredentialID = SearchTagCredentialID
        self.SearchTagMythicTreeID = SearchTagMythicTreeID
        self.SearchTagSource = SearchTagSource
        self.SearchTagData = SearchTagData
        self.SearchTagURL = SearchTagURL
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "search_tag_id": self.SearchTagID,
            "search_tag_task_id": self.SearchTagTaskID,
            "search_tag_file_id": self.SearchTagFileID,
            "search_tag_credential_id": self.SearchTagCredentialID,
            "search_tag_mythictree_id": self.SearchTagMythicTreeID,
            "search_tag_source": self.SearchTagSource,
            "search_tag_data": self.SearchTagData,
            "search_tag_url": self.SearchTagURL
        }


class MythicRPCTagTypeData:
    def __init__(self,
                 id: int = None,
                 name: str = "",
                 description: str = "",
                 color: str = "",
                 operation_id: int = None,
                 **kwargs):
        self.ID = id
        self.Name = name
        self.Description = description
        self.Color = color
        self.OperationID = operation_id
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "id": self.ID,
            "name": self.Name,
            "description": self.Description,
            "color": self.Color,
            "operation_id": self.OperationID
        }


class MythicRPCTagData:
    def __init__(self,
                 id: int = None,
                 tagtype_id: int = None,
                 tagtype: dict = None,
                 data: dict = None,
                 url: str = "",
                 source: str = "",
                 task_id: int = None,
                 file_id: int = None,
                 credential_id: int = None,
                 mythic_tree_id: int = None,
                 **kwargs):
        self.ID = id
        self.TagTypeID = tagtype_id
        self.TagType = MythicRPCTagTypeData(**tagtype) if tagtype is not None else None
        self.Data = data if data is not None else {}
        self.URL = url
        self.Source = source
        self.TaskID = task_id
        self.FileID = file_id
        self.CredentialID = credential_id
        self.MythicTreeID = mythic_tree_id
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "id": self.ID,
            "tagtype_id": self.TagTypeID,
            "tagtype": self.TagType.to_json() if self.TagType is not None else None,
            "data": self.Data,
            "url": self.URL,
            "source": self.Source,
            "task_id": self.TaskID,
            "file_id": self.FileID,
            "credential_id": self.CredentialID,
            "mythic_tree_id": self.MythicTreeID
        }


class MythicRPCTagSearchMessageResponse:
    Tags: list[MythicRPCTagData]

    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 tags: list[dict] = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Tags = [MythicRPCTagData(**x) for x in tags] if tags is not None else []
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCTagSearch(
        msg: MythicRPCTagSearchMessage) -> MythicRPCTagSearchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_TAG_SEARCH,
                                                                            body=msg.to_json())
    return MythicRPCTagSearchMessageResponse(**response)


import mythic_container
from mythic_container.logging import logger
from mythic_container.MythicGoRPC.send_mythic_rpc_callback_search import MythicRPCCallbackSearchMessageResult
import time

MYTHIC_RPC_HANDLE_AGENT_MESSAGE_JSON = "mythic_rpc_handle_agent_message_json"

class MythicRPCHandleAgentMessageJsonMessage:
    """
    Must provide either CallbackID or AgentCallbackID 
    """
    def __init__(self,
                 CallbackID: int = None,
                 AgentCallbackID: str = None,
                 AgentMessage: dict = {},
                 UpdateCheckinTime: bool = False,
                 **kwargs):
        self.CallbackID = CallbackID
        self.AgentCallbackID = AgentCallbackID
        self.AgentMessage = AgentMessage
        self.UpdateCheckinTime = UpdateCheckinTime
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "callback_id": self.CallbackID,
            "agent_callback_id": self.AgentCallbackID,
            "agent_message": self.AgentMessage,
            "update_checkin_time": self.UpdateCheckinTime,
        }

class MythicRPCHandleAgentMessageJsonMessageResponse:

    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 agent_response: dict = {},
                 **kwargs):
        self.Success = success
        self.Error = error
        self.AgentResponse = agent_response
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCHandleAgentMessageJson(
        msg: MythicRPCHandleAgentMessageJsonMessage) -> MythicRPCHandleAgentMessageJsonMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_HANDLE_AGENT_MESSAGE_JSON,
                                                                            body=msg.to_json())
    return MythicRPCHandleAgentMessageJsonMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_EVENTLOG_CREATE = "mythic_rpc_eventlog_create"

MESSAGE_LEVEL_INFO           = "info"
MESSAGE_LEVEL_DEBUG          = "debug"
MESSAGE_LEVEL_AUTH           = "auth"
MESSAGE_LEVEL_AGENT_MESSAGE  = "agent"
MESSAGE_LEVEL_API            = "api"

class MythicRPCOperationEventLogCreateMessage:
    def __init__(self,
                 TaskID: int = None,
                 CallbackID: int = None,
                 AgentCallbackID: str = None,
                 OperationID: int = None,
                 Message: str = None,
                 MessageLevel: str = MESSAGE_LEVEL_INFO,  # info, debug, auth, api, agent
                 Warning: bool = False,
                 **kwargs):
        self.TaskID = TaskID
        self.CallbackID = CallbackID
        self.AgentCallbackID = AgentCallbackID
        self.OperationID = OperationID
        self.Message = Message
        self.MessageLevel = MessageLevel
        self.Warning = Warning
        for k, v in kwargs.items():
            if k == "CallbackId":
                self.CallbackID = v
                logger.warning("MythicRPCOperationEventLogCreateMessage using old API call, update CallbackId to CallbackID")
                continue
            if k == "AgentCallbackId":
                self.AgentCallbackID = v
                logger.warning("MythicRPCOperationEventLogCreateMessage using old API call, update AgentCallbackId to AgentCallbackID")
                continue
            if k == "OperationId":
                self.OperationID = v
                logger.warning("MythicRPCOperationEventLogCreateMessage using old API call, update OperationId to OperationID")
                continue
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "callback_id": self.CallbackID,
            "callback_agent_id": self.AgentCallbackID,
            "operation_id": self.OperationID,
            "message": self.Message,
            "level": self.MessageLevel,
            "warning": self.Warning
        }


class MythicRPCOperationEventLogCreateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCOperationEventLogCreate(
        msg: MythicRPCOperationEventLogCreateMessage) -> MythicRPCOperationEventLogCreateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_EVENTLOG_CREATE,
                                                                            body=msg.to_json())
    return MythicRPCOperationEventLogCreateMessageResponse(*response)



from collections.abc import Callable, Awaitable
from enum import Enum
from typing import List

import mythic_container
from .logging import logger

from .SharedClasses import ContainerOnStartMessage, ContainerOnStartMessageResponse

class ExportFunctionMessage:
    def __init__(self,
                 tree_type: str,
                 container_name: str,
                 host: str,
                 path: str,
                 operation_id: int,
                 operator_id: int,
                 operator_username: str,
                 callback_group: str,):
        self.TreeType = tree_type
        self.ContainerName = container_name
        self.Host = host
        self.Path = path
        self.OperationID = operation_id
        self.OperatorID = operator_id
        self.OperatorUsername = operator_username
        self.CallbackGroup = callback_group
    def to_json(self) -> dict:
        return {
            "tree_type": self.TreeType,
            "container_name": self.ContainerName,
            "host": self.Host,
            "path": self.Path,
            "operation_id": self.OperationID,
            "callback_group": self.CallbackGroup,
            "operator_id": self.OperatorID,
            "operator_username": self.OperatorUsername,
        }
class ExportFunctionMessageResponse:
    def __init__(self,
                 Success: bool = False,
                 Error: str = "",
                 CompletionMessage: str = "",
                 OperationID: int = 0,
                 TreeType: str = "",
                 ):
        self.Success = Success
        self.Error = Error
        self.CompletionMessage = CompletionMessage
        self.OperationID = OperationID
        self.TreeType = TreeType
    def to_json(self) -> dict:
        return {
            "success": self.Success,
            "error": self.Error,
            "completion_message": self.CompletionMessage,
            "operation_id": self.OperationID,
            "tree_type": self.TreeType,
        }
class CustomBrowserTableColumnType(str, Enum):
    String = "string"
    Date = "date"
    Number = "number"
    Size = "size"

class CustomBrowserTableColumn:
    def __init__(self,
                 Key: str,
                 Name: str,
                 FillWidth: bool = False,
                 Width: int = 0,
                 DisableSort: bool = False,
                 DisableDoubleClick: bool = False,
                 DisableFilterMenu: bool = False,
                 ColumnType: CustomBrowserTableColumnType = CustomBrowserTableColumnType.String):
        self.Key = Key
        self.Name = Name
        self.FillWidth = FillWidth
        self.Width = Width
        self.DisableSort = DisableSort
        self.DisableDoubleClick = DisableDoubleClick
        self.DisableFilterMenu = DisableFilterMenu
        self.ColumnType = ColumnType
    def to_json(self) -> dict:
        return {
            "key": self.Key,
            "name": self.Name,
            "fillWidth": self.FillWidth,
            "width": self.Width,
            "disableSort": self.DisableSort,
            "disableDoubleClick": self.DisableDoubleClick,
            "disableFilterMenu": self.DisableFilterMenu,
            "type": self.ColumnType
        }
class CustomBrowserRowAction:
    def __init__(self,
                 Name: str = "",
                 UIFeature: str = "",
                 Icon: str = "",
                 Color: str = "",
                 SupportsFile: bool = False,
                 SupportsFolder: bool = False,
                 OpenDialog: bool = False,
                 GetConfirmation: bool = False):
        self.Name = Name
        self.UIFeature = UIFeature
        self.Icon = Icon
        self.Color = Color
        self.SupportsFile = SupportsFile
        self.SupportsFolder = SupportsFolder
        self.OpenDialog = OpenDialog
        self.GetConfirmation = GetConfirmation
    def to_json(self) -> dict:
        return {
            "name": self.Name,
            "ui_feature": self.UIFeature,
            "icon": self.Icon,
            "color": self.Color,
            "supports_file": self.SupportsFile,
            "supports_folder": self.SupportsFolder,
            "openDialog": self.OpenDialog,
            "getConfirmation": self.GetConfirmation
        }
class CustomBrowserExtraTableTaskingInput:
    def __init__(self,
                 Name: str,
                 Description: str,
                 DisplayName: str,
                 Required: bool = False):
        self.Name = Name
        self.Description = Description
        self.DisplayName = DisplayName
        self.Required = Required
    def to_json(self) -> dict:
        return {
            "name": self.Name,
            "description": self.Description,
            "display_name": self.DisplayName,
            "required": self.Required
        }
class CustomBrowserType(str, Enum):
    """Types of custom browser views available
    """
    File = "file"

class CustomBrowser:
    """The base definition for a custom browser

    """
    name: str = ""
    description: str = ""
    author: str = ""
    semver: str = ""
    type: CustomBrowserType = CustomBrowserType.File
    path_separator: str = ""
    columns: List[CustomBrowserTableColumn] = []
    default_visible_columns: List[str] = []
    indicate_partial_listing: bool = False
    show_current_path: bool = False
    row_actions: List[CustomBrowserRowAction] = []
    extra_table_inputs: List[CustomBrowserExtraTableTaskingInput] = []
    export_function: Callable[
        [ExportFunctionMessage], Awaitable[ExportFunctionMessageResponse]] = None,

    async def on_container_start(self, message: ContainerOnStartMessage) -> ContainerOnStartMessageResponse:
        return ContainerOnStartMessageResponse(ContainerName=self.name)

    def get_sync_message(self):
        return {
            "name": self.name,
            "type": self.type,
            "description": self.description,
            "semver": self.semver,
            "author": self.author,
            "separator": self.path_separator,
            "columns": [x.to_json() for x in self.columns],
            "default_visible_columns": self.default_visible_columns,
            "indicate_partial_listing": self.indicate_partial_listing,
            "show_current_path": self.show_current_path,
            "row_actions": [x.to_json() for x in self.row_actions],
            "extra_table_inputs": [x.to_json() for x in self.extra_table_inputs],
            "export_function": "function exists" if self.export_function is not None else ""
        }


custombrowsers: dict[str, CustomBrowser] = {}


async def SendMythicRPCSyncCustomBrowser(custombrowser_name: str) -> bool:
    try:
        custombrowser_services = CustomBrowser.__subclasses__()
        for cls in custombrowser_services:
            web = cls()
            if web.name == "":
                continue
            if web.name == custombrowser_name:
                custombrowsers.pop(custombrowser_name, None)
                custombrowsers[web.name] = web
                await mythic_container.mythic_service.syncCustomBrowserData(web)
                return True
        return False
    except Exception as e:
        return False



import ujson
from . import CustomBrowserBase
import traceback
import mythic_container


async def ExportFunction(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in CustomBrowserBase.custombrowsers.items():
            if pt.name == msgDict["container_name"]:
                inputMessage = CustomBrowserBase.ExportFunctionMessage(**msgDict)
                if pt.export_function is not None:
                    try:
                        response = await pt.export_function(inputMessage)

                    except Exception as e:
                        response = CustomBrowserBase.ExportFunctionMessageResponse(
                            Success=False,
                            Error=f"Hit exception trying to call export function: {traceback.format_exc()}\n{e}",
                        )
                    response.OperationID = inputMessage.OperationID
                    response.TreeType = inputMessage.TreeType
                    await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                        queue=mythic_container.CUSTOMBROWSER_EXPORT_FUNCTION_RESPONSE,
                        body=response.to_json()
                    )
                else:
                    response = CustomBrowserBase.ExportFunctionMessageResponse(
                        Success=True, CompletionMessage="Export Function not defined",
                        OperationID=inputMessage.OperationID, TreeType=inputMessage.TreeType
                    )
                    await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                        queue=mythic_container.CUSTOMBROWSER_EXPORT_FUNCTION_RESPONSE,
                        body=response.to_json()
                    )
                return
    except Exception as e:
        await mythic_container.RabbitmqConnection.SendDictDirectMessage(
            queue=mythic_container.CUSTOMBROWSER_EXPORT_FUNCTION_RESPONSE,
            body=CustomBrowserBase.ExportFunctionMessageResponse(
                Error=f"Hit exception trying to call export function: {traceback.format_exc()}\n{e}"
            ).to_json()
        )


import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_TAG_CREATE = "mythic_rpc_tag_create"


class MythicRPCTagCreateMessage:
    def __init__(self,
                 TagTypeID: int,
                 URL: str = "",
                 Source: str = "",
                 Data: dict = None,
                 TaskID: int = None,
                 FileID: int = None,
                 CredentialID: int = None,
                 MythicTreeID: int = None,
                 **kwargs):
        self.TagTypeID = TagTypeID
        self.URL = URL
        self.Source = Source
        self.Data = Data if Data is not None else {}
        self.TaskID = TaskID
        self.FileID = FileID
        self.CredentialID = CredentialID
        self.MythicTreeID = MythicTreeID
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "tagtype_id": self.TagTypeID,
            "url": self.URL,
            "source": self.Source,
            "data": self.Data,
            "task_id": self.TaskID,
            "file_id": self.FileID,
            "credential_id": self.CredentialID,
            "mythic_tree_id": self.MythicTreeID
        }


class MythicRPCTagTypeData:
    def __init__(self,
                 id: int = None,
                 name: str = "",
                 description: str = "",
                 color: str = "",
                 operation_id: int = None,
                 **kwargs):
        self.ID = id
        self.Name = name
        self.Description = description
        self.Color = color
        self.OperationID = operation_id
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "id": self.ID,
            "name": self.Name,
            "description": self.Description,
            "color": self.Color,
            "operation_id": self.OperationID
        }


class MythicRPCTagData:
    def __init__(self,
                 id: int = None,
                 tagtype_id: int = None,
                 tagtype: dict = None,
                 data: dict = None,
                 url: str = "",
                 source: str = "",
                 task_id: int = None,
                 file_id: int = None,
                 credential_id: int = None,
                 mythic_tree_id: int = None,
                 **kwargs):
        self.ID = id
        self.TagTypeID = tagtype_id
        self.TagType = MythicRPCTagTypeData(**tagtype) if tagtype is not None else None
        self.Data = data if data is not None else {}
        self.URL = url
        self.Source = source
        self.TaskID = task_id
        self.FileID = file_id
        self.CredentialID = credential_id
        self.MythicTreeID = mythic_tree_id
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "id": self.ID,
            "tagtype_id": self.TagTypeID,
            "tagtype": self.TagType.to_json() if self.TagType is not None else None,
            "data": self.Data,
            "url": self.URL,
            "source": self.Source,
            "task_id": self.TaskID,
            "file_id": self.FileID,
            "credential_id": self.CredentialID,
            "mythic_tree_id": self.MythicTreeID
        }


class MythicRPCTagCreateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 tag: dict = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Tag = MythicRPCTagData(**tag) if tag is not None else None
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCTagCreate(
        msg: MythicRPCTagCreateMessage) -> MythicRPCTagCreateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_TAG_CREATE,
                                                                            body=msg.to_json())
    return MythicRPCTagCreateMessageResponse(**response)


import mythic_container
from mythic_container.logging import logger
from mythic_container.MythicGoRPC.send_mythic_rpc_callback_search import MythicRPCCallbackSearchMessageResult
import time

MYTHIC_RPC_CALLBACK_EDGE_REMOVE = "mythic_rpc_callback_edge_remove"


class MythicRPCCallbackEdgeRemoveMessage:
    def __init__(self,
                 SourceCallbackID: int,
                 DestinationCallbackID: int,
                 C2ProfileName: str,
                 **kwargs):
        self.SourceCallbackID = SourceCallbackID
        self.DestinationCallbackID = DestinationCallbackID
        self.C2ProfileName = C2ProfileName
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "source_callback_id": self.SourceCallbackID,
            "destination_callback_id": self.DestinationCallbackID,
            "c2_profile_name": self.C2ProfileName,
        }

class MythicRPCCallbackEdgeRemoveMessageResponse:

    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCCallbackEdgeRemove(
        msg: MythicRPCCallbackEdgeRemoveMessage) -> MythicRPCCallbackEdgeRemoveMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CALLBACK_EDGE_REMOVE,
                                                                            body=msg.to_json())
    return MythicRPCCallbackEdgeRemoveMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger
import time

MYTHIC_RPC_CUSTOMBROWSER_SEARCH = "mythic_rpc_custombrowser_search"

class MythicRPCCustomBrowserSearchData:
    def __init__(self,
                 TreeType: str,
                 Host: str,
                 Name: str = None,
                 ParentPath: str = None,
                 FullPath: str = None,
                 MetadataKey: str = None,
                 MetadataValue = None,
                 CallbackGroup: str = None,
                 ):
        self.TreeType = TreeType
        self.Host = Host
        self.Name = Name
        self.ParentPath = ParentPath
        self.FullPath = FullPath
        self.MetadataKey = MetadataKey
        self.MetadataValue = MetadataValue
        self.CallbackGroup = CallbackGroup
    def to_json(self):
        return {
            "tree_type": self.TreeType,
            "host": self.Host,
            "name": self.Name,
            "parent_path": self.ParentPath,
            "full_path": self.FullPath,
            "metadata_key": self.MetadataKey,
            "metadata_value": self.MetadataValue,
            "callback_group": self.CallbackGroup
        }
class MythicRPCCustomBrowserSearchMessage:
    def __init__(self,
                 TaskID: int = None,
                 OperationID: int = None,
                 GetAllMatchingChildren: bool = False,
                 SearchCustomBrowser: MythicRPCCustomBrowserSearchData = None):
        self.TaskID = TaskID
        self.OperationID = OperationID
        self.GetAllMatchingChildren = GetAllMatchingChildren
        self.SearchCustomBrowser = SearchCustomBrowser

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "operation_id": self.OperationID,
            "all_matching_children": self.GetAllMatchingChildren,
            "custombrowser": self.SearchCustomBrowser.to_json() if self.SearchCustomBrowser is not None else None
        }
class MythicRPCCustomBrowserSearchDataResponse:
    def __init__(self,
                 tree_type: str,
                 host: str = "",
                 name: str = "",
                 parent_path: str = "",
                 full_path: str = "",
                 metadata: dict = {}):
        self.TreeType = tree_type
        self.Host = host
        self.Name = name
        self.ParentPath = parent_path
        self.FullPath = full_path
        self.Metadata = metadata
    def to_json(self):
        return {
            "tree_type": self.TreeType,
            "host": self.Host,
            "name": self.Name,
            "parent_path": self.ParentPath,
            "full_path": self.FullPath,
            "metadata": self.Metadata
        }
class MythicRPCCustomBrowserSearchMessageResponse:
    CustomBrowser: list[MythicRPCCustomBrowserSearchDataResponse]
    def __init__(self,
                 success: bool = False,
                 error: str = None,
                 custombrowser: list[dict] = None,):
        self.Success = success
        self.Error = error
        self.CustomBrowser = [MythicRPCCustomBrowserSearchDataResponse(**x) for x in custombrowser] if custombrowser is not None else []
    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "custombrowser": [x.to_json() for x in self.CustomBrowser]
        }



async def SendMythicRPCCustomBrowserSearch(
        msg: MythicRPCCustomBrowserSearchMessage) -> MythicRPCCustomBrowserSearchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CUSTOMBROWSER_SEARCH,
                                                                            body=msg.to_json())
    finalResponse = MythicRPCCustomBrowserSearchMessageResponse(**response)
    return finalResponse



import mythic_container
from mythic_container.logging import logger
import time

MYTHIC_RPC_CALLBACKTOKEN_SEARCH = "mythic_rpc_callbacktoken_search"

class MythicRPCCallbackTokenSearchToken:
    def __init__(self,
                 id: int = None,
                 task_id: int = None,
                 deleted: bool = None,
                 host: str = None,
                 description: str = None,
                 operation_id: int = None,
                 timestamp: str = None,
                 token_id: int = None,
                 user: str = None,
                 groups: str = None,
                 privileges: str = None,
                 thread_id: int = None,
                 process_id: int = None,
                 session_id: int = None,
                 logon_sid: str = None,
                 integrity_level_sid: str = None,
                 app_container_sid: str = None,
                 app_container_number: int = None,
                 default_dacl: str = None,
                 restricted: bool = None,
                 handle: int = None,
                 capabilities: str = None,
                 **kwargs):
        self.ID = id
        self.TaskID = task_id
        self.Deleted = deleted
        self.Host = host
        self.Description = description
        self.OperationID = operation_id
        self.Timestamp = timestamp
        self.TokenID = token_id
        self.User = user
        self.Groups = groups
        self.Privileges = privileges
        self.ThreadID = thread_id
        self.ProcessID = process_id
        self.SessionID = session_id
        self.LogonSID = logon_sid
        self.IntegrityLevelSID = integrity_level_sid
        self.AppContainerSID = app_container_sid
        self.AppContainerNumber = app_container_number
        self.DefaultDACL = default_dacl
        self.Restricted = restricted
        self.Handle = handle
        self.Capabilities = capabilities
    def to_json(self):
        return {
            "id": self.ID,
            "task_id": self.TaskID,
            "deleted": self.Deleted,
            "host": self.Host,
            "description": self.Description,
            "operation_id": self.OperationID,
            "timestamp": self.Timestamp,
            "token_id": self.TokenID,
            "user": self.User,
            "groups": self.Groups,
            "privileges": self.Privileges,
            "thread_id": self.ThreadID,
            "process_id": self.ProcessID,
            "session_id": self.SessionID,
            "logon_sid": self.LogonSID,
            "integrity_level_sid": self.IntegrityLevelSID,
            "app_container_sid": self.AppContainerSID,
            "app_container_number": self.AppContainerNumber,
            "default_dacl": self.DefaultDACL,
            "restricted": self.Restricted,
            "handle": self.Handle,
            "capabilities": self.Capabilities,
        }
class MythicRPCCallbackSearchTokenData:
    Token: MythicRPCCallbackTokenSearchToken
    def __init__(self,
                 id: int = None,
                 token: dict = None,
                 callback_id: int = None,
                 task_id: int = None,
                 timestamp_created: str = None,
                 deleted: bool = None,
                 host: str = None,
                 **kwargs):
        self.ID = id
        self.Token = MythicRPCCallbackTokenSearchToken(**token)
        self.CallbackID = callback_id
        self.TaskID = task_id
        self.TimestampCreated = timestamp_created
        self.Deleted = deleted
        self.Host = host
    def to_json(self):
        return {
            "id": self.ID,
            "token": self.Token.to_json() if self.Token else None,
            "callback_id": self.CallbackID,
            "task_id": self.TaskID,
            "timestamp_created": self.TimestampCreated,
            "deleted": self.Deleted,
            "host": self.Host
        }



class MythicRPCCallbackTokenSearchMessage:
    def __init__(self,
                 TaskID: int = None,
                 CallbackID: int = None,
                 AgentCallbackID: str = None):
        self.TaskID = TaskID
        self.CallbackID = CallbackID
        self.AgentCallbackID = AgentCallbackID
    def to_json(self):
        return {
            "task_id": self.TaskID,
            "callback_id": self.CallbackID,
            "agent_callback_id": self.AgentCallbackID
        }


class MythicRPCCallbackTokenSearchMessageResponse:
    def __init__(self,
                 success: bool = None,
                 error: str = None,
                 callbacktokens: list[dict] = None):
        self.Success = success
        self.Error = error
        self.CallbackTokens = [MythicRPCCallbackSearchTokenData(**x) for x in callbacktokens] if callbacktokens else []



async def SendMythicRPCCallbackTokenSearch(
        msg: MythicRPCCallbackTokenSearchMessage) -> MythicRPCCallbackTokenSearchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CALLBACKTOKEN_SEARCH,
                                                                            body=msg.to_json())
    finalResponse = MythicRPCCallbackTokenSearchMessageResponse(**response)
    return finalResponse
